/* 
 * UDP server for Hantek / Tekway / Voltcraft / (Protek?) oscilloscopes
 * 
 * contributors:
 *      
 *      Name                    Date        Changes
 *      David Beinder (yoq)     2012        
 * 
 *
 * This code is licenced under the GPL
 *
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/socket.h>
#include <linux/net.h>
#include <net/sock.h>
#include <asm/processor.h>
#include <asm/uaccess.h>
#include <linux/in.h>

#include "ksocket.h"

#define BUFLEN 10500 //maximum packetsize that can be processed by dso.exe, according to <http://www.mikrocontroller.net/articles/Hantek_Protokoll#0x11_Virtuelle_Remote-Shell> this is 10'239Bytes 
#define PORT 9966

#ifndef MIN
#define MIN( a, b ) ((a)<(b)?(a):(b))
#endif

ksocket_t sockID = NULL;
char readBuf[BUFLEN];
int readPos = 0;
int msgLength = 0;
int msgAvailable = 0;
struct sockaddr_in sender;
int addrLen = sizeof(sender);

void udps_start(void)
{
    struct sockaddr_in listenAddr;
    
    sockID = ksocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (sockID == NULL)
        printk("udpserver: couldn't get socket\n");
        
    memset(&listenAddr, 0, sizeof(listenAddr));
    listenAddr.sin_family = AF_INET;
    listenAddr.sin_port = htons(PORT);
    listenAddr.sin_addr.s_addr = INADDR_ANY;
    
    if (kbind(sockID, (struct sockaddr*)&listenAddr, sizeof(listenAddr)) == -1)
        printk("udpserver: couldn't bind port\n");
    
    printk("udpserver: server running, port bound\n");
}

void udps_stop(void)
{
    if(sockID != NULL)
        kclose(sockID);
}

int udps_checkForMsg(void)
{
    if(msgAvailable)
    {
        return 1;
    }
    else
    {
        msgLength = krecvfrom(sockID, readBuf, 1, MSG_PEEK | MSG_DONTWAIT , (struct sockaddr*)&sender, &addrLen);
        if(msgLength == -EAGAIN)
        {
            return 0; //no packet
        }
        else if(msgLength > 0)
        {
            msgLength = krecvfrom(sockID, readBuf, BUFLEN, 0, (struct sockaddr*)&sender, &addrLen);
            if(msgLength == -1)
            {
                printk("udpserver: error when reading message to buffer\n");
                return 0;
            }
            msgAvailable = 1;
            readPos = 0;
            return 1;
        }
        else
        {
            printk("udpserver: error when peeking for message: %d\n", msgLength);
            return 0;
        }
    }
}

int udps_copyFromRxBuffer(char *destination, int maxBufferSize)
{
    int copyBytes = MIN(maxBufferSize, msgLength - readPos);
    memcpy(destination, &readBuf[readPos], copyBytes);
    readPos += copyBytes;
    
    if(readPos == msgLength)
        msgAvailable = 0;
        
    return copyBytes;
}

int udps_sendResponse(char *buffer, int length)
{
    int result = ksendto(sockID, buffer, length, 0, (struct sockaddr*)&sender, addrLen);
    
    if(result < 0)
    {
        printk("udpserver: sending response failed: %d\n", result);
        return 0;
    }
    
    return result;
}

static int udps_init(void)
{
	printk("udpserver: module loaded\n");
	return 0;
}

static void udps_exit(void)
{
	printk("udpserver: module unloaded\n");
}

module_init(udps_init);
module_exit(udps_exit);

EXPORT_SYMBOL(udps_start);
EXPORT_SYMBOL(udps_stop);
EXPORT_SYMBOL(udps_checkForMsg);
EXPORT_SYMBOL(udps_copyFromRxBuffer);
EXPORT_SYMBOL(udps_sendResponse);

MODULE_LICENSE("GPL");
